module.exports = (context) => {
    let HTML;
    let USERNAME;
    
    const LSKEY = 'codepenUsername';
    const feedParser = require('rss-parser');
    const debounce = require('lodash.debounce');
    const fs = require('fs');
    const path = require('path');
    const pref = context.preferences;

    const commands = [{
        id: 1,
        icon: '#fa fa-code',
        title: 'New Codepen',
        desc: 'Make a new Codepen',
        payload: ['pen-new', 'http://codepen.io/pen/']
    }, {
        id: 1,
        icon: '#fa fa-newspaper-o',
        title: 'New Post',
        desc: 'Make a new Post',
        payload: ['blog-new', 'http://codepen.io/write/']
    }, {
        id: 2,
        icon: '#fa fa-codepen',
        title: 'Your Profile',
        desc: 'Open your Profile in your Browser',
        payload: ['profile-show']
    }, {
        id: 2,
        icon: '#fa fa-user-times',
        title: 'Change CodePen User',
        desc: 'Change your Codepen User',
        payload: ['user-del']
    }];



    function fetchPens() {
        return new Promise((resolve, reject) => {

            feedParser.parseURL('http://codepen.io/' + pref.get('codepenFeed') + '/feed', (err, parsed) => {
                if (err) {
                    reject(err);
                }

                let pens = parsed.feed.entries.map((item) => {
                    let preview = item.link.charAt(0) === '/' ? false : true;
                    let prefix = preview ? '' : 'http:'; 
                    
                    return {
                        id: item.link,
                        icon: '#fa fa-codepen',
                        title: item.title.trim(),
                        desc: item.link,
                        payload: ['view-pen', prefix + item.link],
                        preview: preview
                    };
                });
                resolve(pens);
            });
        });
    }

    function fetchActivities(user) {
        return new Promise((resolve, reject) => {

            feedParser.parseURL('http://codepen.io/' + user + '/activity/feed', (err, parsed) => {
                if (err) {
                    reject(err);
                }

                let activities = parsed.feed.entries.map((item) => {

                    let icoMap = {
                        'hearted': '#fa fa-heart',
                        'followed': '#fa fa-user',
                        'Collection': '#fa fa-th'
                    };


                    function findIcon(title) {
                        for (let find in icoMap) {
                            if (title.search(find) > -1) {
                                return icoMap[find];
                            }
                        }
                    }

                    return {
                        id: item.guid,
                        icon: findIcon(item.title),
                        title: item.title.trim(),
                        desc: item.pubDate,
                        payload: ['link-open', item.guid]
                    };

                });

                resolve(activities);
            });
        });
    }

    function startup() {
        USERNAME = context.localStorage.getItem(LSKEY);
         if ( pref.get('useEmbed') === true) {
            HTML = fs.readFileSync(path.join(__dirname, 'viewembed.html'), 'utf8');
        } else {
            HTML = fs.readFileSync(path.join(__dirname, 'view.html'), 'utf8');
        }
    }

    function search(query, res) {
        const queryTrim = query.trim();

        if (USERNAME === null || typeof USERNAME === 'undefined') {

            res.add({
                id: 0,
                icon: '#fa fa-user',
                title: 'CodePen Username:',
                desc: 'Please type in your CodePen Username and press Enter',
                payload: ['add-user', queryTrim]
            });

        } else {

            if (query.length === 0) {
                res.add(commands.concat({
                    id: 0,
                    icon: '#fa fa-spinner fa-spin',
                    title: 'Loading',
                    desc: 'Fetching Activities for User: ' + USERNAME
                }));

                fetchActivities(USERNAME).then((data) => {
                    res.remove(0);
                    res.add(data);
                }, () => {
                    res.remove(0);
                    res.add({
                        id: 0,
                        icon: '#fa fa-exclamation',
                        title: 'Sorry!',
                        desc: 'Cant find any Activities for User: ' + USERNAME
                    });
                });
            } else {

                    res.add({
                        id: 0,
                        icon: '#fa fa-spinner fa-spin',
                        title: 'Loading',
                        desc: 'Fetching Editors Picks Feed'
                    });

                    let fetchPensDebounced = debounce(() => {
                        fetchPens().then( (data) => {

                            res.remove(0);

                            if (queryTrim.length > 1) {
                                let filtered = context.matchutil.fuzzy(data, queryTrim, x => x.title).map((x) => {
                                    return x.elem;
                                });

                                if (filtered.length > 0) {
                                    res.add(filtered);
                                } else {
                                    res.add({
                                        id: 0,
                                        icon: '#fa fa-exclamation',
                                        title: 'Sorry!',
                                        desc: 'No Pens found with Searchterm: ' + queryTrim
                                    });
                                }
                            } else {
                                res.add(data);
                            }

                    }, () => {
                           res.add({
                                id: 0,
                                icon: '#fa fa-exclamation',
                                title: 'Sorry!',
                                desc: 'Problems with fetching the Editor Picks Feed'
                            });
                        });
                    },500);

                    fetchPensDebounced();
            }
        }
    }

    function execute(id, payload) {

        let [action, data] = payload;

        switch (action) {
            case 'add-user':
                if (data.trim() !== '') {
                    context.localStorage.setItem(LSKEY, data);
                    USERNAME = data;
                    context.app.close();
                }
                break;

            case 'user-del':
                context.localStorage.removeItem(LSKEY);
                USERNAME = null;
                context.app.close();
                break;

            case 'profile-show':
                context.shell.openExternal('http://codepen.io/' + USERNAME);
                context.app.close();
                break;
            default:
                context.shell.openExternal(data);
                context.app.close();
                break;
        }
    }

    function renderPreview(id, payload, render) {
        [http, one, two, dom, user, pen] = id.split('/');   

        let preview = HTML.replace('%pen%', pen);
            preview = preview.replace('%user%', user);
            preview = preview.replace('%link%', id);

        render(preview);
    }

    return {
        startup, search, execute, renderPreview
    };
};